@@ -1,6 +1,6 @@
 /*
- * $Id: di.c,v 2.63 2010-01-31 23:59:34-08 bll Exp $
- * $Source: /home/bll/DI/RCS/di.c,v $
+ * $Id: di.c,v 2.72 2010-03-17 22:44:19-07 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/di.c,v $
  *
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
@@ -109,13 +109,13 @@
 #if _hdr_string
 # include <string.h>
 #endif
-#if _hdr_strings && ((! _hdr_string) || (_include_string))
+#if _hdr_strings
 # include <strings.h>
 #endif
 #if _hdr_memory
 # include <memory.h>
 #endif
-#if _include_malloc && _hdr_malloc
+#if _hdr_malloc
 # include <malloc.h>
 #endif
 #if _hdr_unistd
@@ -307,12 +307,13 @@ typedef struct {
 
 typedef struct {
     int             count;
+    int             haspooledfs;
+    int             totsorted;
     diOptions_t     options;
     diDiskInfo_t    *diskInfo;
     iList_t         ignoreList;
     iList_t         includeList;
     zoneInfo_t      zoneInfo;
-    char            haspooledfs;
 } diData_t;
 
 int          debug = { 0 };
@@ -373,7 +374,7 @@ static void printTitle          _((diOptions_t *));
 static void printPerc           _((_fs_size_t, _fs_size_t, const char *));
 static void processArgs         _((int, char * const [], diData_t *, char *));
 static void setDispBlockSize    _((char *, diOptions_t *));
-static void sortArray           _((diOptions_t *, diDiskInfo_t *, int));
+static void sortArray           _((diOptions_t *, diDiskInfo_t *, int, int));
 static void usage               _((void));
 
 #if defined(__cplusplus)
@@ -399,6 +400,7 @@ main (argc, argv)
         /* initialization */
     diData.count = 0;
     diData.haspooledfs = FALSE;
+    diData.totsorted = FALSE;
 
     diData.diskInfo = (diDiskInfo_t *) NULL;
 
@@ -421,7 +423,7 @@ main (argc, argv)
     diopts->width = 8;
     diopts->inodeWidth = 7;
     diopts->flags = 0;
-#if linux /* Linux loopback works differently than Solaris */
+#if ! SunOS /* Solaris loopback devices should be excluded */
     diopts->flags |= DI_F_INCLUDE_LOOPBACK;
 #endif
     strcpy (diopts->sortType, "m"); /* default - sort by mount point */
@@ -510,6 +512,21 @@ main (argc, argv)
     }
 
     processArgs (argc, argv, &diData, dbsstr);
+    if (debug > 0 && (ptr = getenv ("DI_ARGS")) != (char *) NULL)
+    {
+        printf ("# DI_ARGS:%s\n", ptr);
+    }
+    if (debug > 0)
+    {
+        int j;
+        printf ("# ARGS:");
+        for (j = 0; j < argc; ++j)
+        {
+          printf (" %s", argv[j]);
+        }
+        printf ("\n");
+        printf ("# blocksize: %s\n", dbsstr);
+    }
 
 #if _lib_zone_list && _lib_getzoneid && _lib_zone_getattr
     {
@@ -610,7 +627,7 @@ main (argc, argv)
 
     if (debug > 0)
     {
-        printf ("di ver %s\n", DI_VERSION);
+        printf ("di version %s\n", DI_VERSION);
     }
 
     if (debug > 5)
@@ -645,11 +662,16 @@ main (argc, argv)
     }
 
     preCheckDiskInfo (&diData);
+    if (optind < argc ||
+        (diopts->flags & DI_F_INCLUDE_LOOPBACK) != DI_F_INCLUDE_LOOPBACK)
+    {
+      getDiskStatInfo (&diData);
+      getDiskSpecialInfo (&diData);
+    }
     if (optind < argc)
     {
         int     rc;
 
-        getDiskStatInfo (&diData);
         rc = checkFileInfo (&diData, optind, argc, argv);
         if (rc < 0)
         {
@@ -658,7 +680,6 @@ main (argc, argv)
         }
     }
     di_getDiskInfo (&diData.diskInfo, &diData.count);
-    getDiskSpecialInfo (&diData);
     checkDiskInfo (&diData);
     getMaxFormatLengths (&diData);
     printDiskInfo (&diData);
@@ -734,7 +755,6 @@ printDiskInfo (diData)
     char                lastpool [DI_SPEC_NAME_LEN + 1];
     Size_t              lastpoollen = { 0 };
     int                 inpool = { FALSE };
-    char                tempSortType [DI_SORT_MAX + 1];
 
     lastpool[0] = '\0';
     diopts = &diData->options;
@@ -752,54 +772,35 @@ printDiskInfo (diData)
         printTitle (diopts);
     }
 
+    diskInfo = diData->diskInfo;
     if ((diopts->flags & DI_F_TOTAL) == DI_F_TOTAL)
     {
-            /* need to make a copy of the diskinfo array, as we */
-            /* want to preserve the sort order.                 */
-        diDiskInfo_t         *tempDiskInfo;
         int                  allocated;
 
         allocated = FALSE;
-        if (diData->haspooledfs)
+        if (diData->haspooledfs && ! diData->totsorted)
         {
-              /* in order to find the main pool entry,                */
+          char tempSortType [DI_SORT_MAX + 1];
+              /* in order to find the main pool entries,              */
               /* we must have the array sorted by special device name */
-          tempDiskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) *
-                  (Size_t) diData->count);
-          if (tempDiskInfo == (diDiskInfo_t *) NULL)
-          {
-            fprintf (stderr, "malloc failed in printDiskInfo().  errno %d\n", errno);
-            exit (1);
-          }
-          memcpy ((char *) tempDiskInfo, (char *) diData->diskInfo,
-                  sizeof (diDiskInfo_t) * (Size_t) diData->count);
           strcpy (tempSortType, diopts->sortType);
           strcpy (diopts->sortType, "s");
-          sortArray (diopts, tempDiskInfo, diData->count);
+          sortArray (diopts, diskInfo, diData->count, DI_TOT_SORT_IDX);
           strcpy (diopts->sortType, tempSortType);
-          allocated = TRUE;
-        }
-        else
-        {
-          tempDiskInfo = diData->diskInfo;
+          diData->totsorted = TRUE;
         }
 
         for (i = 0; i < diData->count; ++i)
         {
             diDiskInfo_t    *dinfo;
             int             ispooled;
+            int             startpool;
+            int             poolmain;
 
             ispooled = FALSE;
-            dinfo = &tempDiskInfo [i];
-
-            if (! dinfo->doPrint)
-            {
-              if (debug > 2)
-              {
-                  printf ("tot: %s: not printed; don't add\n", tempDiskInfo[i].name);
-              }
-              continue;
-            }
+            startpool = FALSE;
+            poolmain = FALSE;
+            dinfo = &(diskInfo [diskInfo [i].sortIndex[DI_TOT_SORT_IDX]]);
 
                 /* is it a pooled filesystem type? */
             if (diData->haspooledfs &&
@@ -814,43 +815,49 @@ printDiskInfo (diData)
                 lastpoollen = strlen (lastpool);
                 inpool = FALSE;
               }
+
+              if (strncmp (lastpool, dinfo->special, lastpoollen) == 0)
+              {
+                startpool = TRUE;
+                if (inpool == FALSE)
+                {
+                  poolmain = TRUE;
+                }
+              }
             } else {
               inpool = FALSE;
             }
 
-            if (debug > 2)
+            if (dinfo->doPrint || inpool || poolmain)
             {
-                printf ("tot: %s: add; inpool: %d\n", tempDiskInfo[i].name, inpool);
+              addTotals (dinfo, &totals, inpool);
             }
-            addTotals (dinfo, &totals, inpool);
-
-                /* is it a pooled filesystem type? */
-            if (ispooled)
+            else
             {
-              if (strncmp (lastpool, dinfo->special, lastpoollen) == 0)
+              if (debug > 2)
               {
-                inpool = TRUE;
+                printf ("tot:%s:%s:skip\n", dinfo->special, dinfo->name);
               }
             }
-        } /* for each entry */
 
-        if (allocated)
-        {
-          free ((char *) tempDiskInfo);
-        }
+            if (startpool)
+            {
+              inpool = TRUE;
+            }
+        } /* for each entry */
     } /* if the totals are to be printed */
 
+    diskInfo = diData->diskInfo;
     if (strcmp (diopts->sortType, "n") != 0)
     {
-      sortArray (diopts, diData->diskInfo, diData->count);
+      sortArray (diopts, diskInfo, diData->count, DI_MAIN_SORT_IDX);
     }
 
-    diskInfo = diData->diskInfo;
     for (i = 0; i < diData->count; ++i)
     {
       diDiskInfo_t        *dinfo;
 
-      dinfo = &(diskInfo [diskInfo [i].index]);
+      dinfo = &(diskInfo [diskInfo [i].sortIndex[DI_MAIN_SORT_IDX]]);
       if (debug > 5)
       {
         printf ("pdi:%s:%s:%d:\n", dinfo->name,
@@ -1266,16 +1273,19 @@ addTotals (diskInfo, totals, inpool)
     if (debug > 2)
     {
 #if _siz_long_long >= 8
-        printf ("totals:%s:inp:%d:bs:%lld:mult:%f\n", diskInfo->name,
+        printf ("tot:%s:%s:inp:%d:bs:%lld:mult:%f\n",
+                    diskInfo->special, diskInfo->name,
                     inpool, diskInfo->blockSize, mult);
 #else
-        printf ("totals:%s:inp:%d:bs:%ld:mult:%f\n", diskInfo->name,
+        printf ("tot:%s:%s:inp:%d:bs:%ld:mult:%f\n",
+                    diskInfo->special, diskInfo->name,
                     inpool, diskInfo->blockSize, mult);
 #endif
     }
 
     if (inpool)
     {
+        if (debug > 2) {printf ("  tot:inpool:add total used\n"); }
         /* if in a pool of disks, add the total used to the totals also */
         totals->totalBlocks += (_fs_size_t) ((double) (diskInfo->totalBlocks -
                 diskInfo->freeBlocks) * mult);
@@ -1284,6 +1294,7 @@ addTotals (diskInfo, totals, inpool)
     }
     else
     {
+        if (debug > 2) {printf ("  tot:not inpool:add all totals\n"); }
         totals->totalBlocks += (_fs_size_t) ((double) diskInfo->totalBlocks * mult);
         totals->freeBlocks += (_fs_size_t) ((double) diskInfo->freeBlocks * mult);
         totals->availBlocks += (_fs_size_t) ((double) diskInfo->availBlocks * mult);
@@ -1313,7 +1324,7 @@ printTitle (diopts)
 
     if ((diopts->flags & DI_F_DEBUG_HDR) == DI_F_DEBUG_HDR)
     {
-        printf (DI_GT("di ver %s Default Format: %s\n"),
+        printf (DI_GT("di version %s Default Format: %s\n"),
                 DI_VERSION, DI_DEFAULT_FORMAT);
     }
 
@@ -1528,6 +1539,8 @@ checkFileInfo (diData, optidx, argc, argv)
     int                 i;
     int                 j;
     struct stat         statBuf;
+    diOptions_t         *diopts;
+    diDiskInfo_t        *diskInfo;
 
 
     rc = 0;
@@ -1544,6 +1557,20 @@ checkFileInfo (diData, optidx, argc, argv)
       }
     }
 
+    diopts = &diData->options;
+
+    diskInfo = diData->diskInfo;
+    if (diData->haspooledfs && ! diData->totsorted)
+    {
+      char tempSortType [DI_SORT_MAX + 1];
+
+      strcpy (tempSortType, diopts->sortType);
+      strcpy (diopts->sortType, "s");
+      sortArray (diopts, diskInfo, diData->count, DI_TOT_SORT_IDX);
+      strcpy (diopts->sortType, tempSortType);
+      diData->totsorted = TRUE;
+    }
+
     for (i = optidx; i < argc; ++i)
     {
         int fd;
@@ -1560,22 +1587,102 @@ checkFileInfo (diData, optidx, argc, argv)
         }
         if (src == 0)
         {
+            int             saveIdx;
+            int             found = { FALSE };
+            int             inpool = { FALSE };
+            Size_t          lastpoollen = { 0 };
+            char            lastpool [DI_SPEC_NAME_LEN + 1];
+
             for (j = 0; j < diData->count; ++j)
             {
-                diDiskInfo_t        *dinfo;
+                diDiskInfo_t    *dinfo;
+                int             ispooled;
+                int             startpool;
+                int             poolmain;
 
-                dinfo = &diData->diskInfo[j];
-                if (dinfo->printFlag == DI_PRNT_IGNORE &&
-                    dinfo->st_dev != DI_UNKNOWN_DEV &&
-                    (__ulong) statBuf.st_dev == dinfo->st_dev)
+                ispooled = FALSE;
+                startpool = FALSE;
+                poolmain = FALSE;
+
+                dinfo = &(diskInfo [diskInfo [j].sortIndex[DI_TOT_SORT_IDX]]);
+
+                if (found && ! inpool)
                 {
-                    dinfo->printFlag = DI_PRNT_FORCE;
-                    if (debug > 2)
+                  break;
+                }
+
+                    /* is it a pooled filesystem type? */
+                if (diData->haspooledfs &&
+                    (strcmp (dinfo->fsType, "zfs") == 0 ||
+                     strcmp (dinfo->fsType, "advfs") == 0))
+                {
+                  ispooled = TRUE;
+                  if (lastpoollen == 0 ||
+                      strncmp (lastpool, dinfo->special, lastpoollen) != 0)
+                  {
+                    strncpy (lastpool, dinfo->special, sizeof (lastpool));
+                    lastpoollen = strlen (lastpool);
+                    inpool = FALSE;
+                  }
+
+                  if (strncmp (lastpool, dinfo->special, lastpoollen) == 0)
+                  {
+                    startpool = TRUE;
+                    if (inpool == FALSE)
                     {
-                        printf ("file %s specified: found device %ld (%s)\n",
-                                argv[i], dinfo->st_dev, dinfo->name);
+                      poolmain = TRUE;
                     }
-                    break; /* out of inner for */
+                  }
+                } else {
+                  inpool = FALSE;
+                }
+
+                if (poolmain)
+                {
+                  saveIdx = j;
+                }
+
+                if (found && inpool)
+                {
+                  dinfo = &(diskInfo [diskInfo [j].sortIndex[DI_TOT_SORT_IDX]]);
+                  dinfo->printFlag = DI_PRNT_SKIP;
+                  if (debug > 2)
+                  {
+                    printf ("  inpool B: also process %s %s\n",
+                            dinfo->special, dinfo->name);
+                  }
+                }
+
+                if (dinfo->st_dev != DI_UNKNOWN_DEV &&
+                    (__ulong) statBuf.st_dev == dinfo->st_dev &&
+                    strcmp (dinfo->fsType, "lofs") != 0)
+                {
+                  dinfo->printFlag = DI_PRNT_FORCE;
+                  found = TRUE;
+                  if (debug > 2)
+                  {
+                      printf ("file %s specified: found device %ld (%s %s)\n",
+                              argv[i], dinfo->st_dev, dinfo->special, dinfo->name);
+                  }
+                  if (inpool)
+                  {
+                    int   k;
+                    for (k = saveIdx; k < j; ++k)
+                    {
+                      dinfo = &(diskInfo [diskInfo [k].sortIndex[DI_TOT_SORT_IDX]]);
+                      dinfo->printFlag = DI_PRNT_SKIP;
+                      if (debug > 2)
+                      {
+                        printf ("  inpool A: also process %s %s\n",
+                                dinfo->special, dinfo->name);
+                      }
+                    }
+                  }
+                }
+
+                if (startpool)
+                {
+                  inpool = TRUE;
                 }
             }
         } /* if stat ok */
@@ -1596,12 +1703,13 @@ checkFileInfo (diData, optidx, argc, argv)
  */
 static void
 #if _proto_stdc
-sortArray (diOptions_t *diopts, diDiskInfo_t *data, int count)
+sortArray (diOptions_t *diopts, diDiskInfo_t *data, int count, int sidx)
 #else
-sortArray (diopts, data, count)
+sortArray (diopts, data, count, sidx)
     diOptions_t     *diopts;
     diDiskInfo_t    *data;
     int             count;
+    int             sidx;
 #endif
 {
   unsigned int  tempIndex;
@@ -1624,19 +1732,19 @@ sortArray (diopts, data, count)
   {
     for (i = gap; i < count; ++i)
     {
-      tempIndex = data[i].index;
+      tempIndex = data[i].sortIndex[sidx];
       j = i - gap;
 
-      while (j >= 0 && diCompare (diopts, data, data[j].index, tempIndex) > 0)
+      while (j >= 0 && diCompare (diopts, data, data[j].sortIndex[sidx], tempIndex) > 0)
       {
-        data[j + gap].index = data[j].index;
+        data[j + gap].sortIndex[sidx] = data[j].sortIndex[sidx];
         j -= gap;
       }
 
       j += gap;
       if (j != i)
       {
-        data[j].index = tempIndex;
+        data[j].sortIndex[sidx] = tempIndex;
       }
     }
   }
@@ -1805,8 +1913,8 @@ getDiskSpecialInfo (diData)
             dinfo->sp_rdev = (__ulong) statBuf.st_rdev;
             if (debug > 2)
             {
-                printf ("special dev: %s: %ld rdev: %ld\n",
-                        dinfo->special, dinfo->sp_dev,
+                printf ("special dev: %s %s: %ld rdev: %ld\n",
+                        dinfo->special, dinfo->name, dinfo->sp_dev,
                         dinfo->sp_rdev);
             }
         }
@@ -1837,8 +1945,6 @@ checkDiskInfo (diData)
     int             j;
     int             dupCount;
     _fs_size_t      temp;
-    __ulong         sp_dev;
-    __ulong         sp_rdev;
     diOptions_t     *diopts;
 
 
@@ -1865,14 +1971,15 @@ checkDiskInfo (diData)
         }
 
           /* need to check against include list */
-        if (dinfo->printFlag == DI_PRNT_IGNORE)
+        if (dinfo->printFlag == DI_PRNT_IGNORE ||
+            dinfo->printFlag == DI_PRNT_SKIP)
         {
-            if (debug > 2)
-            {
-                printf ("chk: skipping(%s):%s\n",
-                    getPrintFlagText ((int) dinfo->printFlag), dinfo->name);
-            }
-            dinfo->doPrint = (char) ((diopts->flags & DI_F_ALL) == DI_F_ALL);
+          if (debug > 2)
+          {
+              printf ("chk: skipping(%s):%s\n",
+                  getPrintFlagText ((int) dinfo->printFlag), dinfo->name);
+          }
+          dinfo->doPrint = (char) ((diopts->flags & DI_F_ALL) == DI_F_ALL);
         }
 
             /* Solaris reports a cdrom as having no free blocks,   */
@@ -1942,33 +2049,40 @@ checkDiskInfo (diData)
         diDiskInfo_t        *dinfo;
 
         dinfo = &diData->diskInfo[i];
-        if (dinfo->printFlag == DI_PRNT_IGNORE ||
-            dinfo->printFlag == DI_PRNT_EXCLUDE ||
-            dinfo->printFlag == DI_PRNT_BAD ||
-            dinfo->printFlag == DI_PRNT_OUTOFZONE)
+        if (dinfo->printFlag != DI_PRNT_OK)
         {
-            if (debug > 2)
-            {
-                printf ("dup: chk: skipping(%s):%s\n",
-                    getPrintFlagText ((int) dinfo->printFlag), dinfo->name);
-            }
-            continue;
+          if (debug > 2)
+          {
+              printf ("dup: chk: skipping(%s):%s\n",
+                  getPrintFlagText ((int) dinfo->printFlag), dinfo->name);
+          }
+          continue;
         }
 
             /* don't need to bother checking real partitions */
             /* don't bother if already ignored               */
         if (dinfo->sp_rdev != 0 &&
-            dinfo->printFlag == DI_PRNT_OK)
+            dinfo->printFlag == DI_PRNT_OK &&
+            strcmp (dinfo->fsType, "lofs") == 0)
         {
+          __ulong         sp_dev;
+          __ulong         sp_rdev;
+
           sp_dev = dinfo->sp_dev;
           sp_rdev = dinfo->sp_rdev;
           dupCount = 0;
 
           for (j = 0; j < diData->count; ++j)
           {
-              if (diData->diskInfo [j].sp_dev == sp_rdev)
+              if (diData->diskInfo [j].st_dev == sp_dev)
               {
-                  ++dupCount;
+                if (debug > 2)
+                {
+                  printf ("dup: for %s %ld: found: %s %ld\n",
+                      dinfo->name, sp_dev, diData->diskInfo[j].name,
+                      diData->diskInfo [j].st_dev);
+                }
+                ++dupCount;
               }
           }
 
@@ -1983,10 +2097,10 @@ checkDiskInfo (diData)
             diDiskInfo_t        *dinfob;
 
             dinfob = &diData->diskInfo [j];
-                /* don't reset flags! */
-            if (dinfob->sp_rdev == 0 &&
-                dinfob->sp_dev == sp_rdev &&
-                dinfob->printFlag == DI_PRNT_OK)
+            if (dinfob->printFlag == DI_PRNT_OK &&
+                dinfob->sp_rdev != 0 &&
+                dinfob->st_dev == sp_dev &&
+                strcmp (dinfob->fsType, "lofs") == 0)
             {
               dinfob->printFlag = DI_PRNT_IGNORE;
               dinfob->doPrint = (char) ((diopts->flags & DI_F_ALL) == DI_F_ALL);
@@ -2171,7 +2285,8 @@ preCheckDiskInfo (diData)
         diDiskInfo_t        *dinfo;
 
         dinfo = &diData->diskInfo[i];
-        dinfo->index = (unsigned int) i;
+        dinfo->sortIndex[0] = (unsigned int) i;
+        dinfo->sortIndex[1] = (unsigned int) i;
         if (debug > 4)
         {
             printf ("## prechk:%s:\n", dinfo->name);
@@ -2228,7 +2343,7 @@ usage (void)
 usage ()
 #endif
 {
-    printf (DI_GT("di ver %s    Default Format: %s\n"), DI_VERSION, DI_DEFAULT_FORMAT);
+    printf (DI_GT("di version %s    Default Format: %s\n"), DI_VERSION, DI_DEFAULT_FORMAT);
             /*  12345678901234567890123456789012345678901234567890123456789012345678901234567890 */
     printf (DI_GT("Usage: di [-ant] [-d display-size] [-f format] [-x exclude-fstyp-list]\n"));
     printf (DI_GT("       [-I include-fstyp-list] [file [...]]\n"));
@@ -2521,7 +2636,7 @@ parseList (list, str)
     ocount = list->count;
     list->count += count;
     ncount = list->count;
-    list->list = (char **) Realloc ((char *) list->list,
+    list->list = (char **) _realloc ((char *) list->list,
             list->count * sizeof (char *));
     if (list->list == (char **) NULL)
     {
@@ -2950,5 +3065,6 @@ getPrintFlagText (pf)
         pf == DI_PRNT_IGNORE ? "ignore" :
         pf == DI_PRNT_EXCLUDE ? "exclude" :
         pf == DI_PRNT_OUTOFZONE ? "outofzone" :
-        pf == DI_PRNT_FORCE ? "force" : "unknown";
+        pf == DI_PRNT_FORCE ? "force" :
+        pf == DI_PRNT_SKIP ? "skip" : "unknown";
 }
