@@ -1,6 +1,6 @@
 /*
- * $Id: di.h,v 1.40 2010-01-31 23:21:12-08 bll Exp $
- * $Source: /home/bll/DI/RCS/di.h,v $
+ * $Id: di.h,v 1.44 2010-03-18 01:17:49-07 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/di.h,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
@@ -133,10 +133,14 @@ typedef unsigned long __ulong;
 #define DI_PRNT_OUTOFZONE   3
 #define DI_PRNT_EXCLUDE     4
 #define DI_PRNT_FORCE       5
+#define DI_PRNT_SKIP        6
+
+#define DI_MAIN_SORT_IDX    0
+#define DI_TOT_SORT_IDX     1
 
 typedef struct
 {
-    unsigned int    index;
+    unsigned int    sortIndex [2];
     _fs_size_t      totalBlocks;
     _fs_size_t      freeBlocks;
     _fs_size_t      availBlocks;
@@ -171,7 +175,7 @@ extern int  di_getDiskEntries       _((diDiskInfo_t **, int *));
 extern void di_getDiskInfo          _((diDiskInfo_t **, int *));
  /* getopt.c */
 # if ! _lib_getopt || _npt_getopt
-extern int getopt                   _((char *, char * const [], const char *));
+extern int getopt                   _((int, char * const [], const char *));
 # endif
  /* strdup.c */
 # if ! _lib_strdup
@@ -184,7 +188,7 @@ extern char *strstr                 _((const char *, const char *));
  /* trimchar.c */
 extern void trimChar                _((char *, int));
  /* realloc.c */
-extern void *Realloc                _((void *, Size_t));
+extern void *_realloc               _((void *, Size_t));
  /* didiskutil.c */
 extern void di_initDiskInfo _((diDiskInfo_t *));
 extern void di_saveBlockSizes _((diDiskInfo_t *, _fs_size_t, _fs_size_t, _fs_size_t, _fs_size_t));
@@ -220,4 +224,51 @@ extern int getopt _((int, char * const [], const char *));
 # endif
 #endif
 
+# if _lib_sys_dollar_device_scan && _lib_sys_dollar_getdviw
+
+/* Thanks to Craig Berry's VMS::Device for the VMS code. */
+
+typedef struct {
+  short   buflen;          /* Length of output buffer */
+  short   itmcode;         /* Item code */
+  void    *buffer;         /* Buffer address */
+  void    *retlen;         /* Return length address */
+} VMS_ITMLST;
+
+#define DVI_IS_STRING 1
+#define DVI_IS_LONGWORD 2
+#define DVI_IS_QUADWORD 3
+#define DVI_IS_WORD 4
+#define DVI_IS_BYTE 5
+#define DVI_IS_VMSDATE 6
+#define DVI_IS_BITMAP 7   /* Each bit in the return value indicates something */
+#define DVI_IS_ENUM 8     /* Each returned value has a name, and we ought to */
+                          /* return the name instead of the value */
+#define DVI_IS_ODD 9      /* A catchall */
+
+
+#define DVI_IS_INPUT (1<<0)
+#define DVI_IS_OUTPUT (1<<1)
+#define DVI_ENT(a, b, c) {#a, DVI$_##a, b, c, DVI_IS_OUTPUT}
+
+typedef struct {
+  char *name;         /* Pointer to the item name */
+  int  syscallValue;  /* Value to use in the getDVI item list */
+  int  bufferLen;     /* Length the return va buf needs to be. (no nul */
+                      /* terminators, so must be careful with the return */
+                      /* values. */
+  int  returnType;    /* Type of data the item returns */
+  int  inOut;         /* Is this an input or an output item? */
+} genericID_t ;
+
+/* Macro to fill in a 'traditional' item-list entry */
+#define init_itemlist(ile, length, code, bufaddr, retlen_addr) \
+{ \
+    (ile)->buflen = (length); \
+    (ile)->itmcode = (code); \
+    (ile)->buffer = (bufaddr); \
+    (ile)->retlen = (retlen_addr) ;}
+
+# endif
+
 #endif /* __INC_DI_H_ */
