@@ -1,6 +1,6 @@
 /*
- * $Id: didiskutil.c,v 1.5 2010-01-31 22:02:42-08 bll Exp $
- * $Source: /home/bll/DI/RCS/didiskutil.c,v $
+ * $Id: didiskutil.c,v 1.6 2010-03-12 00:25:15-08 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/didiskutil.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
@@ -15,13 +15,13 @@
 #if _hdr_string
 # include <string.h>
 #endif
-#if _hdr_strings && ((! _hdr_string) || (_include_string))
+#if _hdr_strings
 # include <strings.h>
 #endif
 #if _hdr_memory
 # include <memory.h>
 #endif
-#if _include_malloc && _hdr_malloc
+#if _hdr_malloc
 # include <malloc.h>
 #endif
 #if _hdr_errno
