@@ -1,6 +1,6 @@
 /*
- * $Id: digetentries.c,v 1.8 2010-01-31 21:10:11-08 bll Exp $
- * $Source: /home/bll/DI/RCS/digetentries.c,v $
+ * $Id: digetentries.c,v 1.15 2010-03-20 13:42:22-07 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/digetentries.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
@@ -34,13 +34,13 @@
 #if _hdr_string
 # include <string.h>
 #endif
-#if _hdr_strings && ((! _hdr_string) || (_include_string))
+#if _hdr_strings
 # include <strings.h>
 #endif
 #if _hdr_memory
 # include <memory.h>
 #endif
-#if _include_malloc && _hdr_malloc
+#if _hdr_malloc
 # include <malloc.h>
 #endif
 #if _hdr_time
@@ -115,6 +115,24 @@
 #if _hdr_util_string                /* Syllable */
 # include <util/string.h>           /* os::String - to get mount name */
 #endif
+#if _hdr_starlet                    /* VMS */
+# include <starlet.h>
+#endif
+#if _hdr_descrip                    /* VMS */
+# include <descrip.h>
+#endif
+#if _hdr_dcdef                      /* VMS */
+# include <dcdef.h>
+#endif
+#if _hdr_dvsdef                     /* VMS */
+# include <dvsdef.h>
+#endif
+#if _hdr_ssdef                      /* VMS */
+# include <ssdef.h>
+#endif
+#if _hdr_dvidef                     /* VMS */
+# include <dvidef.h>
+#endif
 
 
 /********************************************************/
@@ -124,9 +142,7 @@
 #endif
 
 #if (_lib_getmntent || \
-    _statfs_2arg || \
-    _statfs_3arg || \
-    _statfs_4arg) && \
+    _statfs_args > 0) && \
     ! _lib_getmntinfo && \
     ! _lib_getfsstat && \
     ! _lib_getvfsstat && \
@@ -201,7 +217,7 @@ di_getDiskEntries (diskInfo, diCount)
     char            *devp;   /* local ptr to dev entry */
 
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: _lib_getmntent\n"); }
+    if (debug > 0) { printf ("# di_getDiskEntries: getmntent\n"); }
     if ((f = fopen (DI_MOUNT_FILE, "r")) == (FILE *) NULL)
     {
         fprintf (stderr, "Unable to open: %s errno %d\n", DI_MOUNT_FILE, errno);
@@ -212,7 +228,7 @@ di_getDiskEntries (diskInfo, diCount)
     {
         idx = *diCount;
         ++*diCount;
-        *diskInfo = (diDiskInfo_t *) Realloc ((char *) *diskInfo,
+        *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
                 sizeof (diDiskInfo_t) * *diCount);
         diptr = *diskInfo + idx;
         di_initDiskInfo (diptr);
@@ -289,7 +305,8 @@ checkMountOptions (mntEntry, str)
     ! _lib_GetDiskFreeSpace && \
     ! _lib_GetDiskFreeSpaceEx && \
     ! _lib_fs_stat_dev && \
-    ! _class_os__Volumes
+    ! _class_os__Volumes && \
+    ! _lib_sys_dollar_device_scan
 
 /*
  * di_getDiskEntries
@@ -312,7 +329,7 @@ di_getDiskEntries (diskInfo, diCount)
     struct mnttab    mntEntry;
 
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: not anything; sys v.3\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: not anything; sys v.3\n"); }
     if ((f = fopen (DI_MOUNT_FILE, "r")) == (FILE *) NULL)
     {
         fprintf (stderr, "Unable to open: %s errno %d\n", DI_MOUNT_FILE, errno);
@@ -328,7 +345,7 @@ di_getDiskEntries (diskInfo, diCount)
         {
             idx = *diCount;
             ++*diCount;
-            *diskInfo = (diDiskInfo_t *) Realloc ((char *) *diskInfo,
+            *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
                     sizeof (diDiskInfo_t) * *diCount);
             di_initDiskInfo (diptr);
 
@@ -398,9 +415,9 @@ di_getDiskEntries (diskInfo, diCount)
     char            *devp;   /* local ptr to dev entry */
 
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: set/get/endmntent\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: set/get/endmntent\n"); }
 /* if both are set not an ansi compiler... */
-#if _setmntent_1arg && ! _setmntent_2arg
+#if _setmntent_args == 1
     if ((f = setmntent (DI_MOUNT_FILE)) == (FILE *) NULL)
 #else
     if ((f = setmntent (DI_MOUNT_FILE, "r")) == (FILE *) NULL)
@@ -414,7 +431,7 @@ di_getDiskEntries (diskInfo, diCount)
     {
         idx = *diCount;
         ++*diCount;
-        *diskInfo = (diDiskInfo_t *) Realloc ((char *) *diskInfo,
+        *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
                 sizeof (diDiskInfo_t) * (Size_t) *diCount);
         diptr = *diskInfo + idx;
         di_initDiskInfo (diptr);
@@ -510,7 +527,7 @@ di_getDiskEntries (diskInfo, diCount)
     short           fstype;
     struct statfs   *mntbufp;
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: getmntinfo\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: getmntinfo\n"); }
     count = getmntinfo (&mntbufp, MNT_WAIT);
     if (count < 1)
     {
@@ -790,7 +807,7 @@ di_getDiskEntries (diskInfo, diCount)
     struct statfs   *mntbufp;
     struct statfs   *sp;
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: getfsstat\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: getfsstat\n"); }
     count = getfsstat ((struct statfs *) NULL, 0, MNT_NOWAIT);
     if (count < 1)
     {
@@ -935,7 +952,7 @@ di_getDiskEntries (diskInfo, diCount)
     struct statvfs  *mntbufp;
     struct statvfs  *sp;
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: getvfsstat\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: getvfsstat\n"); }
     count = getvfsstat ((struct statvfs *) NULL, 0, ST_NOWAIT);
     if (count < 1)
     {
@@ -1059,7 +1076,7 @@ di_getDiskEntries (diskInfo, diCount)
     int             len;
 
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: getmnt\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: getmnt\n"); }
     bufsize = NMOUNT * sizeof (struct fs_data);  /* enough for max # mounts */
     fsdbuf = (struct fs_data *) malloc (bufsize);
     if (fsdbuf == (struct fs_data *) NULL)
@@ -1210,7 +1227,7 @@ di_getDiskEntries (diskInfo, diCount)
     int             len;
 
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: mntctl\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: mntctl\n"); }
     i = 0;
     vmbufsz = sizeof (struct vmount) * DI_FSMAGIC; /* initial vmount buffer */
 
@@ -1364,10 +1381,10 @@ di_getDiskEntries (diskInfo, diCount)
 
 
 # if _lib_GetDiskFreeSpaceEx
-    if (debug > 0) { printf ("# lib:getDiskEntries: GetDiskFreeSpaceEx\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: GetDiskFreeSpaceEx\n"); }
 # else
 #  if _lib_GetDiskFreeSpace
-    if (debug > 0) { printf ("# lib:getDiskEntries: GetDiskFreeSpace\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: GetDiskFreeSpace\n"); }
 #  endif
 # endif
     diskflag = DI_PRNT_IGNORE;
@@ -1447,7 +1464,7 @@ di_getDiskEntries (diskInfo, diCount)
     BEntry          entry;
     BPath           path;
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: fs_stat_dev\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: fs_stat_dev\n"); }
     count = 0;
     while ((dev = next_dev (&count)) != B_BAD_VALUE)
     {
@@ -1458,7 +1475,7 @@ di_getDiskEntries (diskInfo, diCount)
 
         idx = *diCount;
         ++*diCount;
-        *diskInfo = (diDiskInfo_t *) Realloc ((char *) *diskInfo,
+        *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
                 sizeof (diDiskInfo_t) * *diCount);
         diptr = *diskInfo + idx;
         di_initDiskInfo (diptr);
@@ -1540,7 +1557,7 @@ di_getDiskEntries (diskInfo, diCount)
     os::Volumes     *vols;
     os::String      mpString;
 
-    if (debug > 0) { printf ("# lib:getDiskEntries: os::Volumes\n"); }
+    if (debug > 0) { printf ("# getDiskEntries: os::Volumes\n"); }
     vols = new os::Volumes;
     count = vols->GetMountPointCount ();
     for (i = 0; i < count; ++i)
@@ -1552,7 +1569,7 @@ di_getDiskEntries (diskInfo, diCount)
 
         idx = *diCount;
         ++*diCount;
-        *diskInfo = (diDiskInfo_t *) Realloc ((char *) *diskInfo,
+        *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
                 sizeof (diDiskInfo_t) * *diCount);
         diptr = *diskInfo + idx;
         di_initDiskInfo (diptr);
@@ -1605,4 +1622,377 @@ di_getDiskEntries (diskInfo, diCount)
 
 #endif
 
+/*
+ * VMS
+ */
+
+#if _lib_sys_dollar_device_scan && _lib_sys_dollar_getdviw
+
+/* Thanks to Craig Berry's VMS::Device for the VMS code. */
+
+char *monthNames[12] = {
+  "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep",
+  "Oct", "Nov", "Dec" };
+
+genericID_t devInfoList[] =
+{
+#ifdef DVI$_DISPLAY_DEVNAM
+  DVI_ENT(DISPLAY_DEVNAM, 256, DVI_IS_STRING),
+#endif
+#ifdef DVI$_DEVBUFSIZ
+  DVI_ENT(DEVBUFSIZ, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_MAXBLOCK
+  DVI_ENT(MAXBLOCK, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_FREEBLOCKS
+  DVI_ENT(FREEBLOCKS, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_MAXFILES
+  DVI_ENT(MAXFILES, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_REMOTE_DEVICE
+  DVI_ENT(REMOTE_DEVICE, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_VOLNAM
+  DVI_ENT(VOLNAM, 12, DVI_IS_STRING),
+#endif
+#ifdef DVI$_AVL
+  DVI_ENT(AVL, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_MNT
+  DVI_ENT(MNT, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_SHR
+  DVI_ENT(SHR, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_ODS2_SUBSET0
+  DVI_ENT(ODS2_SUBSET0, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_ODS5
+  DVI_ENT(ODS5, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_NET
+  DVI_ENT(NET, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_DEVICE_TYPE_NAME
+  DVI_ENT(DEVICE_TYPE_NAME, 64, DVI_IS_STRING),
+#endif
+#ifdef DVI$_SHDW_MASTER
+  DVI_ENT(SHDW_MASTER, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_SHDW_MEMBER
+  DVI_ENT(SHDW_MEMBER, 4, DVI_IS_LONGWORD),
+#endif
+#ifdef DVI$_SHDW_MASTER_NAME
+  DVI_ENT(SHDW_MASTER_NAME, 64, DVI_IS_STRING),
+#endif
+  {NULL, 0, 0, 0}
+};
+#define devInfoListCount ((sizeof(devInfoList)/sizeof (genericID_t)) - 1)
+
+
+int
+# if _proto_stdc
+di_getDiskEntries (diDiskInfo_t **diskInfo, int *diCount)
+# else
+di_getDiskEntries (diskInfo, diCount)
+    diDiskInfo_t **diskInfo;
+    int *diCount;
+# endif
+{
+  diDiskInfo_t    *diptr;
+  int             idx;
+  unsigned short  devNameLen;
+  unsigned short  devReturnLen;
+  unsigned int    deviceClass;
+  char            context[8] = {0,0,0,0,0,0,0,0};
+  struct dsc$descriptor_s devNameDesc;
+  struct dsc$descriptor_s devSearchNameDesc;
+  VMS_ITMLST      scanItemList;
+  VMS_ITMLST      *itemList;
+  unsigned short  *returnLengths;
+  char            **returnBuffers;
+  long            *tempLong;
+  char            returnedDevName [65];
+  unsigned short  returnedTime[7];
+  char            displayTime[100];
+  int             i;
+  int             status;
+# ifdef __ALPHA
+  __int64         *tempLongLong;
+# endif
+  _fs_size_t      freeBlocks;
+  _fs_size_t      totBlocks;
+  _fs_size_t      maxFiles;
+  _fs_size_t      tblocksz;
+
+
+  if (debug > 0) { printf ("# getDiskEntries: sys$device_scan/sys$getdviw\n"); }
+
+  devNameDesc.dsc$a_pointer = returnedDevName;
+  devNameDesc.dsc$w_length = 64;
+  devNameDesc.dsc$b_dtype = DSC$K_DTYPE_T;
+  devNameDesc.dsc$b_class = DSC$K_CLASS_S;
+
+  devSearchNameDesc.dsc$a_pointer = "*";
+  devSearchNameDesc.dsc$w_length = 1;
+  devSearchNameDesc.dsc$b_dtype = DSC$K_DTYPE_T;
+  devSearchNameDesc.dsc$b_class = DSC$K_CLASS_S;
+
+  memset (&scanItemList, 0, sizeof(VMS_ITMLST));
+  itemList = malloc (sizeof(VMS_ITMLST) * (devInfoListCount));
+  memset (itemList, 0, sizeof(VMS_ITMLST) * (devInfoListCount));
+  returnBuffers = (char **) malloc (sizeof (char *) * devInfoListCount);
+  returnLengths = malloc (sizeof(short) * devInfoListCount);
+
+  for (i = 0; i < devInfoListCount; i++) {
+    returnBuffers[i] = malloc (devInfoList[i].bufferLen + 1);
+    memset (returnBuffers[i], 0, devInfoList[i].bufferLen + 1);
+    init_itemlist (&itemList[i], devInfoList[i].bufferLen,
+          devInfoList[i].syscallValue, returnBuffers[i],
+          &returnLengths[i]);
+  }
+
+  deviceClass = DC$_DISK;
+  init_itemlist (&scanItemList, sizeof(deviceClass),
+     DVS$_DEVCLASS, &deviceClass, NULL);
+
+  while (sys$device_scan (&devNameDesc, &devReturnLen,
+          &devSearchNameDesc, &scanItemList, context) == SS$_NORMAL) {
+    idx = *diCount;
+    ++*diCount;
+    *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
+        sizeof (diDiskInfo_t) * (Size_t) *diCount);
+    diptr = *diskInfo + idx;
+    di_initDiskInfo (diptr);
+
+    returnedDevName [devReturnLen] = '\0';
+
+    if (debug > 4)
+    {
+      printf ("%s\n", returnedDevName);
+    }
+
+    devNameDesc.dsc$a_pointer = returnedDevName;
+    devNameDesc.dsc$w_length = devReturnLen;
+    devNameDesc.dsc$b_dtype = DSC$K_DTYPE_T;
+    devNameDesc.dsc$b_class = DSC$K_CLASS_S;
+
+    strncpy (diptr->fsType, "ODS-2", DI_TYPE_LEN);
+    /* _$9$LDA2: */
+    if (strlen (returnedDevName) > 4 && returnedDevName[4] == 'L')
+    {
+      strncpy (diptr->fsType, "LD", DI_TYPE_LEN);
+    }
+
+    status = sys$getdviw(0, 0, &devNameDesc, itemList,
+            NULL, NULL, NULL, NULL);
+    if (status == SS$_NORMAL) {
+      for (i = 0; i < devInfoListCount; i++) {
+        switch (devInfoList[i].returnType) {
+          case DVI_IS_STRING:
+            if (debug > 4)
+            {
+              printf ("  string:");
+              printf ("%s:", devInfoList[i].name);
+              printf ("%*s:\n", (int) returnLengths[i],
+                      (char *) returnBuffers[i]);
+            }
+            if (strcmp (devInfoList[i].name, "DISPLAY_DEVNAM") == 0)
+            {
+                strncpy (diptr->special, returnBuffers[i], returnLengths[i]);
+                diptr->special[returnLengths[i]] = '\0';
+            }
+            if (strcmp (devInfoList[i].name, "VOLNAM") == 0)
+            {
+                strncpy (diptr->name, returnBuffers[i], returnLengths[i]);
+                diptr->name[returnLengths[i]] = '\0';
+            }
+            if (strcmp (devInfoList[i].name, "DEVICE_TYPE_NAME") == 0)
+            {
+              returnBuffers[i][returnLengths[i]] = '\0';
+              strncat (diptr->options, "dev=",
+                  DI_OPT_LEN - strlen (diptr->options) - 1);
+              strncat (diptr->options, returnBuffers[i],
+                  DI_OPT_LEN - strlen (diptr->options) - 1);
+              strncat (diptr->options, ",",
+                  DI_OPT_LEN - strlen (diptr->options) - 1);
+            }
+            if (strcmp (devInfoList[i].name, "SHDW_MASTER_NAME") == 0)
+            {
+              if (returnLengths[i] > 0) {
+                returnBuffers[i][returnLengths[i]] = '\0';
+                strncat (diptr->options, returnBuffers[i],
+                    DI_OPT_LEN - strlen (diptr->options) - 1);
+                strncat (diptr->options, "),",
+                    DI_OPT_LEN - strlen (diptr->options) - 1);
+              }
+            }
+            break;
+          case DVI_IS_VMSDATE:
+            sys$numtim (returnedTime, returnBuffers[i]);
+            sprintf (displayTime, "%02hi-%s-%hi %02hi:%02hi:%02hi.%02hi",
+                    returnedTime[2], monthNames[returnedTime[1] - 1],
+                    returnedTime[0], returnedTime[3], returnedTime[4],
+                    returnedTime[5], returnedTime[6]);
+            if (debug > 4)
+            {
+              printf ("  date:");
+              printf ("%s:", devInfoList[i].name);
+              printf ("%s:\n", displayTime);
+            }
+            break;
+          case DVI_IS_ENUM:
+            tempLong = (long *) returnBuffers[i];
+            if (debug > 4)
+            {
+              printf ("  enum:");
+              printf ("%s:", devInfoList[i].name);
+              printf ("%ld:\n", *tempLong);
+            }
+            break;
+          case DVI_IS_BITMAP:
+          case DVI_IS_LONGWORD:
+            tempLong = (long *) returnBuffers[i];
+            if (debug > 4)
+            {
+              printf ("  long:");
+              printf ("%s:", devInfoList[i].name);
+              printf ("%ld:\n", *tempLong);
+            }
+            if (strcmp (devInfoList[i].name, "MAXBLOCK") == 0)
+            {
+                totBlocks = *tempLong;
+            }
+            if (strcmp (devInfoList[i].name, "FREEBLOCKS") == 0)
+            {
+                freeBlocks = *tempLong;
+            }
+            if (strcmp (devInfoList[i].name, "MAXFILES") == 0)
+            {
+                maxFiles = *tempLong;
+            }
+            if (strcmp (devInfoList[i].name, "REMOTE_DEVICE") == 0)
+            {
+                if (*tempLong == 1L)
+                {
+                    diptr->isLocal = FALSE;
+                }
+            }
+            if (strcmp (devInfoList[i].name, "NET") == 0)
+            {
+                if (*tempLong == 1L)
+                {
+                    diptr->isLocal = FALSE;
+                }
+            }
+            if (strcmp (devInfoList[i].name, "DEVBUFSIZ") == 0)
+            {
+                tblocksz = *tempLong;
+            }
+            if (strcmp (devInfoList[i].name, "AVL") == 0)
+            {
+              if (*tempLong == 0L)
+              {
+                diptr->printFlag = DI_PRNT_IGNORE;
+                strncat (diptr->options, "Offline,",
+                    DI_OPT_LEN - strlen (diptr->options) - 1);
+              }
+              if (*tempLong == 1L)
+              {
+                strncat (diptr->options, "Online,",
+                    DI_OPT_LEN - strlen (diptr->options) - 1);
+              }
+            }
+            if (strcmp (devInfoList[i].name, "MNT") == 0)
+            {
+              if (*tempLong == 1L)
+              {
+                strncat (diptr->options, "Mounted,",
+                    DI_OPT_LEN - strlen (diptr->options) - 1);
+              }
+              else
+              {
+                diptr->printFlag = DI_PRNT_IGNORE;
+              }
+            }
+            if (strcmp (devInfoList[i].name, "SHR") == 0)
+            {
+              if (*tempLong == 1L)
+              {
+                strncat (diptr->options, "Shareable,",
+                    DI_OPT_LEN - strlen (diptr->options) - 1);
+              }
+            }
+            if (strcmp (devInfoList[i].name, "SHDW_MASTER") == 0)
+            {
+              if (*tempLong == 1L)
+              {
+                strncat (diptr->options, "Shadow Master,",
+                    DI_OPT_LEN - strlen (diptr->options) - 1);
+              }
+            }
+            if (strcmp (devInfoList[i].name, "SHDW_MEMBER") == 0)
+            {
+              if (*tempLong == 1L)
+              {
+                strncat (diptr->options, "Shadow Member(",
+                    DI_OPT_LEN - strlen (diptr->options) - 1);
+              }
+            }
+            if (strcmp (devInfoList[i].name, "ODS2_SUBSET0") == 0)
+            {
+              if (*tempLong == 1L)
+              {
+                strncpy (diptr->fsType, "ODS-2/0", DI_TYPE_LEN);
+              }
+            }
+            if (strcmp (devInfoList[i].name, "ODS5") == 0)
+            {
+              if (*tempLong == 1L)
+              {
+                strncpy (diptr->fsType, "ODS-5", DI_TYPE_LEN);
+              }
+            }
+            break;
+# ifdef __ALPHA
+          case DVI_IS_QUADWORD:
+            tempLongLong = (__int64 *) returnBuffers[i];
+            if (debug > 4)
+            {
+              printf ("  quad:");
+              printf ("%s:", devInfoList[i].name);
+              printf ("%llu:\n", (long long) *tempLongLong);
+            }
+            break;
+# endif
+          }
+      }
+    } else {
+      /* fail */
+      ;
+    }
+
+    di_saveBlockSizes (diptr, tblocksz,
+          totBlocks, freeBlocks, freeBlocks);
+    di_saveInodeSizes (diptr, maxFiles, 0, 0);
+
+    /* reset this so there's room during the next syscall */
+    devNameDesc.dsc$w_length = 64;
+    trimChar (diptr->options, ',');
+  } /* while there are entries */
+
+  for (i = 0; i < devInfoListCount; i++) {
+    free (returnBuffers[i]);
+  }
+  free (returnBuffers);
+  free (returnLengths);
+  free (itemList);
+
+  return 0;
+}
+
+#endif
+
 
