@@ -1,6 +1,6 @@
 /*
- * $Id: digetinfo.c,v 1.7 2010-01-31 21:09:43-08 bll Exp $
- * $Source: /home/bll/DI/RCS/digetinfo.c,v $
+ * $Id: digetinfo.c,v 1.12 2010-03-14 16:41:48-07 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/digetinfo.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
@@ -38,7 +38,7 @@
 #if _hdr_string
 # include <string.h>
 #endif
-#if _hdr_strings && ((! _hdr_string) || (_include_string))
+#if _hdr_strings
 # include <strings.h>
 #endif
 
@@ -72,13 +72,13 @@
 #if ! _lib_statvfs && \
 	_lib_statfs && \
 	_npt_statfs
-# if _lib_statfs && _statfs_2arg
+# if _lib_statfs && _statfs_args == 2
   extern int statfs _((char *, struct statfs *));
 # endif
-# if _lib_statfs && _statfs_3arg
+# if _lib_statfs && _statfs_args == 3
   extern int statfs _((char *, struct statfs *, int));
 # endif
-# if _lib_statfs && _statfs_4arg
+# if _lib_statfs && _statfs_args == 4
   extern int statfs _((char *, struct statfs *, int, int));
 # endif
 #endif
@@ -121,12 +121,13 @@ di_getDiskInfo (diskInfo, diCount)
     int             i;
     Statvfs_t       statBuf;
 
-    if (debug > 0) { printf ("# lib:getDiskInfo: statvfs\n"); }
+    if (debug > 0) { printf ("# getDiskInfo: statvfs\n"); }
     for (i = 0; i < *diCount; ++i)
     {
         diptr = *diskInfo + i;
 
         if (diptr->printFlag == DI_PRNT_OK ||
+            diptr->printFlag == DI_PRNT_SKIP ||
             diptr->printFlag == DI_PRNT_FORCE)
         {
             _fs_size_t      tblocksz;
@@ -164,14 +165,20 @@ di_getDiskInfo (diskInfo, diCount)
                             (long) statBuf.f_frsize);
 # if _siz_long_long >= 8
                     printf ("\tblocks: tot:%llu free:%lld avail:%llu\n",
-                           statBuf.f_blocks, statBuf.f_bfree, statBuf.f_bavail);
+                        (long long) statBuf.f_blocks,
+                        (long long) statBuf.f_bfree,
+                        (long long) statBuf.f_bavail);
                     printf ("\tinodes: tot:%llu free:%llu avail:%llu\n",
-                            statBuf.f_files, statBuf.f_ffree, statBuf.f_favail);
+                        (long long) statBuf.f_files,
+                        (long long) statBuf.f_ffree,
+                        (long long) statBuf.f_favail);
 # else
                     printf ("\tblocks: tot:%lu free:%lu avail:%lu\n",
-                           statBuf.f_blocks, statBuf.f_bfree, statBuf.f_bavail);
+                        (long) statBuf.f_blocks, (long) statBuf.f_bfree,
+                        (long) statBuf.f_bavail);
                     printf ("\tinodes: tot:%lu free:%lu avail:%lu\n",
-                            statBuf.f_files, statBuf.f_ffree, statBuf.f_favail);
+                        (long) statBuf.f_files, (long) statBuf.f_ffree,
+                        (long) statBuf.f_favail);
 # endif
                 }
             }
@@ -186,7 +193,7 @@ di_getDiskInfo (diskInfo, diCount)
 
 #endif /* _lib_statvfs */
 
-#if _lib_statfs && _statfs_4arg && \
+#if _lib_statfs && _statfs_args == 4 && \
     ! _lib_statvfs && \
     ! _lib_getmntinfo && \
     ! _lib_getfsstat && \
@@ -224,11 +231,12 @@ di_getDiskInfo (diskInfo, diCount)
     int             i;
     struct statfs   statBuf;
 
-    if (debug > 0) { printf ("# lib:getDiskInfo: sysv-statfs 4arg\n"); }
+    if (debug > 0) { printf ("# getDiskInfo: sysv-statfs 4arg\n"); }
     for (i = 0; i < *diCount; ++i)
     {
         diptr = *diskInfo + i;
         if (diptr->printFlag == DI_PRNT_OK ||
+            diptr->printFlag == DI_PRNT_SKIP ||
             diptr->printFlag == DI_PRNT_FORCE)
         {
             _fs_size_t      tblocksz;
@@ -283,9 +291,9 @@ di_getDiskInfo (diskInfo, diCount)
     } /* for each entry */
 }
 
-#endif /* _statfs_4arg */
+#endif /* _statfs_args == 4 */
 
-#if _lib_statfs && (_statfs_2arg ||_statfs_3arg) && \
+#if _lib_statfs && (_statfs_args == 2 || _statfs_args == 3) && \
         ! _lib_statvfs && \
         ! _lib_getmntinfo && \
         ! _lib_getfsstat && \
@@ -315,11 +323,12 @@ di_getDiskInfo (diskInfo, diCount)
     int             i;
     struct statfs   statBuf;
 
-    if (debug > 0) { printf ("# lib:getDiskInfo: bsd-statfs 2/3arg\n"); }
+    if (debug > 0) { printf ("# getDiskInfo: bsd-statfs 2/3arg\n"); }
     for (i = 0; i < *diCount; ++i)
     {
         diptr = *diskInfo + i;
         if (diptr->printFlag == DI_PRNT_OK ||
+            diptr->printFlag == DI_PRNT_SKIP ||
             diptr->printFlag == DI_PRNT_FORCE)
         {
             if (statfs (diptr->name, &statBuf) == 0)
@@ -357,7 +366,7 @@ di_getDiskInfo (diskInfo, diCount)
     } /* for each entry */
 }
 
-#endif /* _statfs_2arg */
+#endif /* _statfs_args == 2 or 3 */
 
 
 #if _lib_GetVolumeInformation
@@ -392,11 +401,12 @@ di_getDiskInfo (diskInfo, diCount)
     DWORD               fsFlags;
 
 
-    if (debug > 0) { printf ("# lib:getDiskInfo: GetVolumeInformation\n"); }
+    if (debug > 0) { printf ("# getDiskInfo: GetVolumeInformation\n"); }
     for (i = 0; i < *diCount; ++i)
     {
         diptr = *diskInfo + i;
         if (diptr->printFlag == DI_PRNT_OK ||
+            diptr->printFlag == DI_PRNT_SKIP ||
             diptr->printFlag == DI_PRNT_FORCE)
         {
             rc = GetVolumeInformation (diptr->name,
@@ -506,7 +516,7 @@ di_getDiskInfo (diskInfo, diCount)
     int *diCount;
 #  endif
 {
-    if (debug > 0) { printf ("# lib:getDiskInfo: empty\n"); }
+    if (debug > 0) { printf ("# getDiskInfo: empty\n"); }
     return;
 }
 #endif
