@@ -1,6 +1,6 @@
 /*
-$Id: getopt.c,v 1.12 2010-01-31 22:02:49-08 bll Exp $
-$Source: /home/bll/DI/RCS/getopt.c,v $
+$Id: getopt.c,v 1.13 2010-03-12 00:25:33-08 bll Exp $
+$Source: /home/bll/DI/di/RCS/getopt.c,v $
 */
 
 #include "config.h"
@@ -9,7 +9,7 @@ $Source: /home/bll/DI/RCS/getopt.c,v $
 
 /*
  *
- * $Id: getopt.c,v 1.12 2010-01-31 22:02:49-08 bll Exp $
+ * $Id: getopt.c,v 1.13 2010-03-12 00:25:33-08 bll Exp $
  *
  * getopt - get option letter from argv
  *
@@ -24,7 +24,7 @@ $Source: /home/bll/DI/RCS/getopt.c,v $
 #if _hdr_string
 # include <string.h>
 #endif
-#if _hdr_strings && ((! _hdr_string) || (_include_string))
+#if _hdr_strings
 # include <strings.h>
 #endif
 
