@@ -1,6 +1,6 @@
 /*
- * $Id: realloc.c,v 1.4 2010-01-31 11:29:48-08 bll Exp $
- * $Source: /home/bll/DI/RCS/realloc.c,v $
+ * $Id: realloc.c,v 1.6 2010-03-12 00:25:37-08 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/realloc.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
@@ -14,12 +14,11 @@
 #if _hdr_memory
 # include <memory.h>
 #endif
-#if _include_malloc && _hdr_malloc
+#if _hdr_malloc
 # include <malloc.h>
 #endif
 
 /*
- * Realloc
  *
  * portable realloc
  * some variants don't accept a null pointer for initial allocation.
@@ -28,9 +27,9 @@
 
 void *
 #if _proto_stdc
-Realloc (void *ptr, Size_t size)
+_realloc (void *ptr, Size_t size)
 #else
-Realloc (ptr, size)
+_realloc (ptr, size)
     void        *ptr;
     Size_t      size;
 #endif
