@@ -1,6 +1,6 @@
 /*
- * $Id: strdup.c,v 1.3 2010-01-31 22:02:51-08 bll Exp $
- * $Source: /home/bll/DI/RCS/strdup.c,v $
+ * $Id: strdup.c,v 1.4 2010-03-12 00:25:42-08 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/strdup.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
@@ -17,13 +17,13 @@
 # if _hdr_memory
 #  include <memory.h>
 # endif
-# if _include_malloc && _hdr_malloc
+# if _hdr_malloc
 #  include <malloc.h>
 # endif
 # if _hdr_string
 #  include <string.h>
 # endif
-# if _hdr_strings && ((! _hdr_string) || (_include_string))
+# if _hdr_strings
 #  include <strings.h>
 # endif
 
