@@ -1,6 +1,6 @@
 /*
- * $Id: strstr.c,v 1.3 2010-01-31 22:02:56-08 bll Exp $
- * $Source: /home/bll/DI/RCS/strstr.c,v $
+ * $Id: strstr.c,v 1.4 2010-03-12 00:25:46-08 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/strstr.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
@@ -17,7 +17,7 @@
 # if _hdr_string
 #  include <string.h>
 # endif
-# if _hdr_strings && ((! _hdr_string) || (_include_string))
+# if _hdr_strings
 #  include <strings.h>
 # endif
 
