@@ -1,6 +1,6 @@
 /*
- * $Id: trimchar.c,v 1.5 2010-01-31 21:07:48-08 bll Exp $
- * $Source: /home/bll/DI/RCS/trimchar.c,v $
+ * $Id: trimchar.c,v 1.6 2010-03-12 00:25:52-08 bll Exp $
+ * $Source: /home/bll/DI/di/RCS/trimchar.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
@@ -14,7 +14,7 @@
 #if _hdr_string
 # include <string.h>
 #endif
-#if _hdr_strings && ((! _hdr_string) || (_include_string))
+#if _hdr_strings
 # include <strings.h>
 #endif
 
